import java.util.Properties;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.jms.*;
import javax.rmi.PortableRemoteObject;

public class JMSClient
{
	public static void main(String[] args)
	{
		try
		{
			Subscriber subscriber = new Subscriber("TYPE = 'CACHE'");
			Thread thread = new Thread(subscriber);
			thread.start();
			thread.join();
		}
		catch (Exception x) {x.printStackTrace();}
	}

	private static class Subscriber implements Runnable
	{
		private double m_rate;
		private final long m_interval = 1000;
		private Listener m_listener;
		private String m_selector;
		private Subscriber(String selector) {m_selector = selector;}
		public void run()
		{
			TopicConnection connection = null;
			try
			{
				java.io.InputStream jndi = getClass().getResourceAsStream("/defaultjndi.properties");
				Properties p = new Properties();
				p.load(jndi);
				Context context = new InitialContext(p);
				Object refFactory = context.lookup("TopicConnectionFactory");
				TopicConnectionFactory factory =(TopicConnectionFactory)PortableRemoteObject.narrow(refFactory, TopicConnectionFactory.class);
				connection = factory.createTopicConnection();
				TopicSession session = connection.createTopicSession(false, Session.DUPS_OK_ACKNOWLEDGE);

				Object refTopic = context.lookup("topic/beancache");
				Topic topic = (Topic)PortableRemoteObject.narrow(refTopic, Topic.class);

				TopicSubscriber subscriber = session.createSubscriber(topic, m_selector, false);
				m_listener = new Listener();
				subscriber.setMessageListener(m_listener);

				connection.start();

				while (true)
				{
					Thread.sleep(m_interval);
					processMessages();
					System.out.println("Rate of messages with selector '" + m_selector + "': " + getMessageRate() + " messages per second.");
				}
			}
			catch (Exception x)
			{
				x.printStackTrace();
				if (connection != null) {try {connection.close();} catch (JMSException ignored) {}}
			}
		}
		private void processMessages()
		{
			long seconds = m_interval / 1000;
			m_rate = (double)m_listener.m_count / seconds;
			m_listener.m_count = 0;
		}
		private double getMessageRate() {return m_rate;}
	}
	private static class Listener implements MessageListener
	{
		public int m_count;
		public void onMessage(Message message)
		{
			++m_count;
		}
	}
}